@@ -62,6 +62,7 @@ PERFORMANCE OF THIS SOFTWARE.
 #include "kernel.h"
 #include "interfaces.h"
 #include "struct.h"
+#include "util_funcs.h"
 
 netsnmp_feature_child_of(get_routes, libnetsnmpmibs)
 
@@ -1463,8 +1464,8 @@ qsort_compare(const void *v1, const void *v2)
 #endif                          /* solaris2 */
 
 #elif defined(HAVE_IPHLPAPI_H)  /* WIN32 cygwin */
-#define VIEWSTATUS VACMVIEWSTATUS
-#undef VACMVIEWSTATUS
+#define VACMVIEWSTATUS VIEWSTATUS
+#undef VIEWSTATUS
 #include <iphlpapi.h>
 #ifndef MIB_IPPROTO_NETMGMT
 #define MIB_IPPROTO_NETMGMT 3
